@@ -46,14 +46,14 @@ summary.mWaveD <- function(object, ...){
   cat('Blur type: ',object$blurType,'\n\n')
   cat("Estimated Channel information:\n\n")
   
-  if( object$blurType == "direct" ){
+  if (object$blurType == "direct") {
     mat <- cbind(round(object$sigma, 3), round(object$alpha, 3), object$blurInfo$freq, rep(object$j1, object$channels))
     colnames(mat) <- c("Sigma.hat", "Alpha", "Fourier freq cutoff", "Highest resolution")
     rownames(mat) <- paste("Channel ", 1:object$channels,':', sep='')
     print(mat, ...)
   } else {
     # If Smooth blur is used, display the matrix of values
-    if( object$blurType == "smooth"){
+    if (object$blurType == "smooth"){
       mat <- cbind(round(object$sigma, 3), round(object$alpha, 3), object$blurInfo$freq, object$blurInfo$maxLevels)
       colnames(mat) <- c("Sigma.hat", "Alpha", "Fourier freq cutoff", "Highest resolution")
       rownames(mat) <- paste("Channel ", 1:object$channels,':', sep='')
@@ -62,7 +62,7 @@ summary.mWaveD <- function(object, ...){
       cat("Estimated best channel = Channel", object$blurInfo$bestChannel)
       
     } else {
-      if( object$blurType == "box.car"){
+      if (object$blurType == "box.car"){
         mat <- cbind(round(object$sigma, 3), round(object$alpha, 3))
         colnames(mat) <- c("Sigma.hat", "Alpha")
         rownames(mat) <- paste("Channel ", 1:object$channels,':', sep='')
@@ -90,40 +90,39 @@ summary.mWaveD <- function(object, ...){
 #' @description Plots the wavelet coefficient object in the multiresolution analysis
 #' 
 #' @param x A list of class waveletCoef.
+#' @param y An optional numeric vector of trimmed wavelet coefficients to be overlayed on top of the plot for comparison with the \code{x} wavelet coefficients. 
+#' @param labels Optional character vector with two elements to give name labels to \code{x} and \code{y} respectively.
 #' @param ... Arguments to be passed to methods.
 #' @param lowest Specifies the coarsest resolution to display in the Multi-resolution plot.
 #' @param highest Specifies the finest resolution to display in the Multi-resolution plot.
-#' @param shrinkCoef A numeric vector of trimmed wavelet coefficients to be overlayed on top of the plot for comparison with the 'x' wavelet coefficients. 
-#' @param thickness An integer that specifies the thickness of the overlayed coefficients (shrinkCoef values) in the plot. Larger values increase the thickness.
-#' @param descending A logical value to specify whether resolutions on the y-axis of the plot are increasing from top to bottom.
 #' @param scaling A numeric value that acts as a graphical scaling parameter to rescale the wavelet coefficients in the plot. A larger scaling value will reduce the size of the coefficients in the plot.
+#' @param ggplot A logical value to specify if the user wants to use base graphics (FALSE) or ggplot2  graphics (TRUE).
 #' 
 #' @seealso \code{\link{multiCoef}} for generating a list of class `waveletCoef`
 #' 
 #' @export
-plot.waveletCoef <- function(x, ..., lowest = NULL, highest = NULL, shrinkCoef = NULL, thickness = 3, descending = FALSE, scaling = 1){
-  if ( !is.null(shrinkCoef) && class(shrinkCoef) != "waveletCoef" ){
-    stop('shrinkCoef must be a waveletCoef object')
+plot.waveletCoef <- function(x, y = NULL, labels = NULL,  ..., lowest = NULL, highest = NULL, scaling = 1, ggplot = TRUE){
+  if (!is.null(y) && class(y) != "waveletCoef") {
+    stop('y argument must be a waveletCoef object')
   }
-  def.par <- par(no.readonly = TRUE)
   
-  J <- floor(log2(length(x$coef)))  
+  J <- floor(log2(length(x$coef)))
   fine <- ceiling(J) - 1
   # Check resolution ranges
-  if (is.null(lowest)){
+  if (is.null(lowest)) {
     lowest <- x$j0
   } else {
     # Catch lowest level too low
-    if( lowest > x$j0 )
-      warning('lowest level shouldnt be smaller than j0 specified in wavelet coefficient object.')
+    if (lowest > x$j0 )
+      warning("lowest level shouldn't be smaller than j0 specified in wavelet coefficient object.")
   }
   if (is.null(highest)) {
     highest <- fine
   } else {
-    if( highest > fine ){
-      warning(paste('highest level too high. Restting highest level to the maximum at j1 = ', fine))
+    if (highest > fine) {
+      warning(paste('highest level too high. Resetting highest level to the maximum at j1 = ', fine))
     } else {
-      if( highest < lowest ){
+      if (highest < lowest) {
         warning('highest level must be higher than the lowest level.')
         highest <- lowest
       }
@@ -133,39 +132,70 @@ plot.waveletCoef <- function(x, ..., lowest = NULL, highest = NULL, shrinkCoef =
   js <- rep(lowest:highest, 2^(lowest:highest))
   ks <- unlist(lapply(lowest:highest, function(i) 0:(2^i-1)/2^i))
   wi <- (2^lowest + 1):2^(highest + 1)
+  nw <- length(wi)
   w  <- x$coef[wi]*scaling
-  wf <- 2 * max(abs(w))/scaling
+  wf <- 2.05 * max(abs(w))/scaling
   w  <- w/wf
   ws <- w + js
   
-  if( !is.null(shrinkCoef)){
-    j0Trim <- shrinkCoef$j0
-    if( j0Trim != x$j0){
-      warning('shrinkCoef object has a different coarse resolution j0 than x, interpret lower resolution levels with caution')
+  # check shrink input is a waveletCoef object
+  if (!is.null(y)) {
+    if (length(x$coef) != length(y$coef)) {
+      stop('length of y coefficients is different to the length of x coefficients')
     }
-    wShrink <- shrinkCoef$coef[wi]/wf
+    j0Trim <- y$j0
+    if (j0Trim != x$j0) {
+      warning('y object has a different coarse resolution j0 than x object, interpret lower resolution levels with caution')
+    }
+    wShrink <- y$coef[wi]/wf
     survived <- which(wShrink != 0)
     kss <- ks[survived]
     jss <- js[survived]
     wss <- wShrink[survived] + jss
-  } 
-  buf <- 0.5
-  plot(0, type = "n", xlim = c(0,1), ylim = c(lowest - buf, highest + buf), yaxt = 'n', xlab = "Location",
-       ylab = "Resolution Level", main = "MultiResolution Analysis of Coef.")
-  axis(2, at = lowest:highest)
-  if( !is.null(shrinkCoef) ){
-    col = 'red'
+    ns <- 2
+    nss <- length(survived)
   } else {
-    col = 1
+    ns <- 1
   }
   
-  segments(ks, js, ks, ws, lwd = thickness, col = col)
+  mraTitle <- 'Multiresolution Analysis of Coef.'
+  mraLabels <- c("Location", "Resolution Level")
+  if (!is.null(labels)) {
+    if (length(labels) != ns) {
+      warning('length of labels might not be long enough.')
+      labels = c('x', 'y')
+    }
+  } else {
+    labels = c('x', 'y')
+  }
   
-  if( !is.null(shrinkCoef)){
-    segments(kss, jss, kss, wss, lwd = thickness + 2, col = 'blue')
+  if (ggplot && require(ggplot2)) {
+    if (ns == 2) {
+      nData <- data.frame(w = c(ws,wss), js = c(js, jss), ks = c(ks, kss), col = rep(labels, c(nw, nss)))
+      mraPlot <- ggplot(nData) + geom_segment(aes(x = ks, xend = ks, y = js, yend = w, colour = col, size = col)) + labs(x = mraLabels[1], y = mraLabels[2]) + scale_size_discrete(range = c(1, 2))
+      # Fix legend
+      mraPlot <- mraPlot + theme(legend.position = "top", axis.text.y = element_text(angle = 90)) + guides(colour = guide_legend(title = mraTitle), size = guide_legend(title = mraTitle))
+    } else {
+      nData <- data.frame(w = ws, js = js, ks = ks)
+      mraPlot <- ggplot(nData) + geom_segment(aes(x = ks, xend = ks, y = js, yend = w), colour = 'red') + ggtitle(mraTitle)
+    }
+    mraPlot + labs(x = mraLabels[1], y = mraLabels[2]) + scale_y_continuous(breaks = lowest:highest) 
+  } else {
+    buf <- 0.5
+    plot(0, type = "n", xlim = c(0,1), ylim = c(lowest - buf, highest + buf), yaxt = 'n', xlab = mraLabels[1], ylab = mraLabels[2], main = mraTitle)
+    axis(2, at = lowest:highest)
+    if (!is.null(y)) {
+      col = 'red'
+    } else {
+      col = 1
+    }
+    
+    segments(ks, js, ks, ws, col = col)
+    
+    if (!is.null(y)) {
+      segments(kss, jss, kss, wss, lwd = 2, col = 'blue')
+    }
   }
-  # return to default layout
-  par(def.par)
 }
 
 #' @name plot.mWaveD
@@ -178,8 +208,7 @@ plot.waveletCoef <- function(x, ..., lowest = NULL, highest = NULL, shrinkCoef =
 #' @param which A numeric vector that specifies which plots to output. Default value is \code{1:4} which  specifies that all four plots are to be displayed.
 #' @param ask A logical value that specifies whether the user is \emph{ask}ed before each plot output.
 #' @param singlePlot A logical value that controls whether all plots should appear on a single window. The plot window is resized depending the value of \code{which}.
-#' @param thickness An integer that specifies the thickness of the overlayed coefficients (shrinkCoef values) in the plot. Larger values increase the thickness.
-#' @param ggplot A logical value to specify if the user wants to use base graphics (FALSE) or ggplot2 (TRUE) graphics.
+#' @param ggplot A logical value to specify if the user wants to use base graphics (FALSE) or ggplot2 graphics (TRUE).
 #' 
 #' @details Four plots are output that summarise the multichannel input, a visualisation of the characteristics of the channels and the output estimate and a multi-resolution analysis plot.\itemize{
 #' \item Plot 1: Multichannel input signal overlayed.
@@ -193,19 +222,19 @@ plot.waveletCoef <- function(x, ..., lowest = NULL, highest = NULL, shrinkCoef =
 #' Wishart, J.R. (2014) \emph{Data-driven wavelet resolution choice in multichannel box-car deconvolution with long memory}, Proceedings of COMPSTAT 2014, Geneva Switzerland, Physica Verlag, Heidelberg (to appear)
 #' @seealso \code{\link{multiWaveD}}
 #' @export
-plot.mWaveD <- function(x, ..., which = 1L:4L, singlePlot = TRUE, ask = !singlePlot, thickness = 3, ggplot = TRUE){
-  par.def <- par(no.readonly = TRUE)
-  showPrompt <- function(condition){
-    if (condition){
+plot.mWaveD <- function(x, ..., which = 1L:4L, singlePlot = TRUE, ask = !singlePlot, ggplot = TRUE){
+  showPrompt <- function(condition) {
+    if (condition) {
       readline('Press any key to see the next plot.')  
     }
   }
+  
   # Check if ggplot is available if requested
-  if (ggplot){
-    ggAvailable <- 'ggplot2' %in% .packages(all.available = TRUE)
+  if (ggplot) {
+    ggAvailable <- require(ggplot2)
     # Check optional dependency
-    gridExtraAvailable <- 'gridExtra' %in% .packages(all.available = TRUE)
-    if (ggAvailable){
+    gridExtraAvailable <- require(gridExtra)
+    if (ggAvailable) {
       hsize <- 1
       lsize <- 0.5
       asize <- 0.5
@@ -214,8 +243,8 @@ plot.mWaveD <- function(x, ..., which = 1L:4L, singlePlot = TRUE, ask = !singleP
       ggList <- list(NULL)
       i <- 1
     } 
-    if (singlePlot){
-      if (gridExtraAvailable){
+    if (singlePlot) {
+      if (gridExtraAvailable) {
         library(gridExtra)
       } else {
         warning('gridExtra package required to create ggplot2 graphics in same window. Setting output to separate windows.')
@@ -228,13 +257,13 @@ plot.mWaveD <- function(x, ..., which = 1L:4L, singlePlot = TRUE, ask = !singleP
   
   show <- rep(FALSE, 4)
   # Make sure which argument is numeric
-  if (!is.numeric(which)){
+  if (!is.numeric(which)) {
     stop('`which` argument must be a vector containing numerical elements')
   }
   # Only consider the integer bits between 1:4
   which <- which[which %in% 1L:4L]
   # Complain if there are no values in 1:4
-  if (length(which) == 0){
+  if (length(which) == 0) {
     stop('`which` argument must be a vector containing elements: 1, 2, 3 or 4')
   }
   show[which] <- TRUE
@@ -257,8 +286,8 @@ plot.mWaveD <- function(x, ..., which = 1L:4L, singlePlot = TRUE, ask = !singleP
   mraLabels <- c('raw',paste(x$shrinkType, ' thresholded', sep = ''))
   mraTitle <- 'Multiresolution Analysis'
   
-  if (show[3L]){
-    if (blurType != "box.car"){
+  if (show[3L]) {
+    if (blurType != "box.car") {
       blurf <- blurInfo$decay
       cutf <- blurInfo$cutoff
       ymin <- min(cutf)
@@ -268,7 +297,7 @@ plot.mWaveD <- function(x, ..., which = 1L:4L, singlePlot = TRUE, ask = !singleP
       blur <- rbind(revblur, blurf)
       cut <- rbind(revcut, cutf)
       ylim <- c(min(cutf[2, ]), 0)
-      if (blurType == 'smooth'){
+      if (blurType == 'smooth') {
         xbest <- max(blurInfo$freqCutoffs) - 1
         ybest <- blurf[xbest, blurInfo$bestChannel]
         xlim <- min(2*max(blurInfo$freqCutoff), n/2)
@@ -284,23 +313,23 @@ plot.mWaveD <- function(x, ..., which = 1L:4L, singlePlot = TRUE, ask = !singleP
       ylim <- range(c(blurInfo$blockVar, blurInfo$blockCutoff))
     }
   }
-  if (show[1L] && ggAvailable){
+  if (show[1L] && ggAvailable) {
     signalData <- data.frame(Y = as.vector(x$signal), x = rep(t, m), Channel = rep(LETTERS[1:m], each = n))
     signalPlot <- ggplot(signalData, aes_string(x = 'x', y = 'Y', colour = 'Channel')) + geom_line(size = lsize, alpha = asize) + ggtitle(signalTitle) + labs(x = '', y = '')
     ggList[[i]] <- signalPlot
     i <- i + 1
   }
-  if (show[2L] && ggAvailable){
+  if (show[2L] && ggAvailable) {
     estimateData <- data.frame(Y = as.vector(x$estimate), x = t)
     estimatePlot <- ggplot(estimateData, aes_string(x = 'x', y = 'Y')) + geom_line(size = lsize, alpha = asize) + ggtitle(estimateTitle) + labs(x = '', y = '')
     ggList[[i]] <- estimatePlot
     i <- i + 1
   }
-  if (show[3L] && ggAvailable){
-    if (blurType != 'box.car'){
+  if (show[3L] && ggAvailable) {
+    if (blurType != 'box.car') {
       fourierData <- data.frame(Y = as.vector(blur), x = rep(iw,m), Ycut = as.vector(cut), Channel=rep(LETTERS[1:m],each=n),m=m)
       resolutionPlot <- ggplot(fourierData) + geom_line(aes_string(x = 'x', y = 'Y', colour = 'Channel', group = 'Channel'),size = 1) + geom_line(aes_string(x = 'x', y = 'Ycut', colour = 'Channel'), linetype='dashed', size = 1) + ggtitle(fourierTitle) + labs(x = fourierLabel, y = '')
-      if (blurType == 'smooth'){
+      if (blurType == 'smooth') {
         highlightData <- data.frame(x = rep(xbest, 2), y = c(-Inf, ybest))
         pointData <- data.frame(xbest = xbest, ybest = ybest)
         resolutionPlot <- resolutionPlot + geom_line(aes_string(x = 'x', y = 'y'), linetype = 'dotted' , data = highlightData) + geom_line(aes_string(x = 'x', y = 'y'), linetype = 'dotted' , data = data.frame(x = rep(-xbest + 1, 2), y = c(-Inf, ybest))) + geom_point( aes_string(x = 'xbest', y = 'ybest'), size = 4, shape = 1, data = pointData) + geom_point( aes_string(x = 'x', y = 'y'), size = 4, shape = 1, data = data.frame(x = -xbest + 1, y = ybest)) + coord_cartesian(xlim = xlim)
@@ -316,83 +345,63 @@ plot.mWaveD <- function(x, ..., which = 1L:4L, singlePlot = TRUE, ask = !singleP
     i <- i + 1
   }
   
-  if (show[4L]){
-    # MRA plot
-    js <- rep(j0:j1, 2^(j0:j1))
-    ks <- unlist(lapply(j0:j1, function(i) (0:(2^i-1)/2^i)))
-    wi <- (2^j0 + 1):2^(j1 + 1)
-    nw <- length(wi)
-    w  <- x$coef$coef[wi]
-    wf <- 2.05 * max(w)
-    w  <- w/wf
-    wc <- x$shrinkCoef$coef[wi]/wf
-    
-    if (ggAvailable){
-      raw.size <- 1
-      shrink.size <- 1
-      
-      mraData <- data.frame(w = c(w + js, wc+js), js = rep(js, 2), ks = rep(ks, 2), MRA = rep(LETTERS[1:2], each = nw), wsize = rep(c(raw.size, shrink.size), each = nw), method = rep(mraLabels, each = nw))
-      
-      mraPlot <- ggplot(mraData) + geom_segment(aes_string(x = 'ks', y = 'js', xend = 'ks', yend = 'w', colour = 'MRA', size = 'wsize')) + labs(x = "", y = 'j') + scale_color_discrete(labels = mraLabels, guide = guide_legend(title.position = 'left', title.theme = element_text(size = 15, angle = 0))) + scale_size(guide = 'none') + guides(colour = guide_legend(override.aes = list(size = 10), title = mraTitle)) + theme(legend.position = "top", legend.key=element_rect(fill = NA), axis.text.y = element_text(angle = 90))  + scale_y_continuous(breaks = j0:j1)
+  if (show[4L] && ggAvailable) {
+      mraPlot <- plot(x$coef, x$shrinkCoef, highest = j1, labels = c('Raw', paste(x$shrinkType, ' Thresholding', sep = '')), ggplot = TRUE)
       ggList[[i]] <- mraPlot
-    }
   }
 
-  if (ask){
+  if (ask) {
     oask <- devAskNewPage(TRUE)
     on.exit(devAskNewPage(oask))
   }
   
-  if (ggAvailable){
+  if (ggAvailable) {
     # Plot them
-    if (singlePlot == TRUE){
+    if (singlePlot == TRUE) {
       do.call(grid.arrange, ggList)  
     } else {
-      if (show[1L]){
+      if (show[1L]) {
         print(signalPlot)
       }
-      if (show[2L]){
+      if (show[2L]) {
 #         showPrompt(ask)
         print(estimatePlot)
       }
-      if (show[3L]){
+      if (show[3L]) {
 #         showPrompt(ask)
         print(resolutionPlot)
       }
-      if (show[4L]){
+      if (show[4L]) {
 #         showPrompt(ask)
         print(mraPlot)
       }
     }
   } else {
-    if (singlePlot){
-      plotDims <- switch(sum(show),
-                         c(1, 1), c(2, 1), c(3, 1), c(2, 2))
+    if (singlePlot) {
+      plotDims <- switch(sum(show), c(1, 1), c(2, 1), c(3, 1), c(2, 2))
       par(mfrow = plotDims)
     } else {
       par(mfrow = c(1,1))
     }
 
-    if (show[1L]){
+    if (show[1L]) {
       matplot(t, x$signal, type = 'l', main = signalTitle, ylab = '', xlab = '', lty = 1, cex = 0.8)
       grid()
     }
 
-    if (show[2L]){
-#       showPrompt(ask)
+    if (show[2L]) {
       # Plot mWaveD estimate
       plot(t, x$estimate, type = 'l', main = estimateTitle, ylab = '', xlab = '')
       grid()
     }
 
-    if (show[3L]){
+    if (show[3L]) {
       # Plot resolution analysis
-      if( blurType != 'box.car'){
-  #       showPrompt(ask)
+      if (blurType != 'box.car') {
         matplot(iw, blur, type = 'l', lty = 1, xlim = xlim, ylim = ylim, main = fourierTitle, xlab = fourierLabel, ylab = "")
         matlines(iw, cut, lty = 2)
         grid()      
-        if (blurType == 'smooth'){
+        if (blurType == 'smooth') {
           points(xbest, ybest, col='blue')
           points(-xbest + 1, ybest, col = 'blue')
           xbest <- rep(xbest, 2)
@@ -410,12 +419,10 @@ plot.mWaveD <- function(x, ..., which = 1L:4L, singlePlot = TRUE, ask = !singleP
       }
     }
 #     showPrompt(ask)
-    if (show[4L]){
-      plot(x$coef, highest = j1, shrinkCoef = x$shrinkCoef, ...)
+    if (show[4L]) {
+      plot(x$coef, x$shrinkCoef, highest = j1, ..., ggplot = FALSE)
     }
   }
-  # Return to default
-  par(par.def)
 }
 
 #' @name mWaveDDemo
